#ifndef BUILDER_H_
#define BUILDER_H_

#include <list>
#include <map>
#include <string>
#include <msp/datafile/loader.h>
#include <msp/fs/path.h>
#include "architecture.h"
#include "buildgraph.h"
#include "buildtype.h"
#include "config.h"
#include "logger.h"
#include "packagemanager.h"
#include "target.h"
#include "toolchain.h"
#include "virtualfilesystem.h"

class FileTarget;
class Package;
class SourcePackage;

/**
This class ties everything else together.  It also contains code for loading
build files and supervising the build process.
*/
class Builder
{
private:
	class Loader: public Msp::DataFile::ObjectLoader<Builder>
	{
	private:
		const Config::InputOptions *options;
		bool conf_all;

	public:
		Loader(Builder &, const Config::InputOptions * = 0, bool = false);
		~Loader();

	private:
		void architecture(const std::string &);
		void binpkg(const std::string &);
		void build_type(const std::string &);
		void package(const std::string &);
	};

private:
	typedef std::map<std::string, BuildType> BuildTypeMap;

	PackageManager package_manager;

	Architecture native_arch;
	Architecture *current_arch;
	BuildTypeMap build_types;
	BuildType *build_type;
	Toolchain toolchain;
	VirtualFileSystem vfs;
	BuildGraph build_graph;
	Logger default_logger;
	const Logger *logger;

	Msp::FS::Path prefix;
	Msp::FS::Path tempdir;

	Loader *top_loader;

public:
	Builder();
	~Builder();

	PackageManager &get_package_manager() { return package_manager; }

	void set_architecture(const std::string &);
	const Architecture &get_current_arch() const { return *current_arch; }
	const Architecture &get_native_arch() const { return native_arch; }
	void set_build_type(const std::string &);
	const BuildType &get_build_type() const { return *build_type; }
	BuildGraph &get_build_graph() { return build_graph; }
	void set_prefix(const Msp::FS::Path &);
	void set_temp_directory(const Msp::FS::Path &);
	const Msp::FS::Path &get_prefix() const { return prefix; }
	const Msp::FS::Path &get_temp_directory() const { return tempdir; }

	void add_default_tools();
	const Toolchain &get_toolchain() const { return toolchain; }
	VirtualFileSystem &get_vfs() { return vfs; }
	void set_logger(const Logger *);
	const Logger &get_logger() const { return *logger; }

	std::list<std::string> collect_problems() const;

	/** Loads a build file.  If opts is not null, it is used to configure any
	packages loaded from this file.  If all is true, external packages are also
	configured. */
	void load_build_file(const Msp::FS::Path &, const Config::InputOptions *opts = 0, bool all = false);

	/** Builds the goal targets.  The build graph must be prepared first. */
	int build(unsigned jobs = 1, bool dry_run = false, bool show_progress = false);

	/** Cleans buildable targets.  If all is true, cleans all packages.
	Otherwise cleans only the default package. */
	int clean(bool all = false, bool dry_run = false);

	int do_create_makefile();
};

#endif
